/****************************************************************************
**  MDHashTool - Message Digest (MD5, SHA-1, etc.) tool
**  Copyright (c) 2005  Charles Melhorn
**
**  This file is part of MDHashTool.
**
**  MDHashTool is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  MDHashTool is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  Or see the GNU web site at: http://www.gnu.org
**
****************************************************************************/ 

// used to create file input stream - from nsUpdateService.js
const MODE_RDONLY   = 0x01;
const PERMS_FILE    = 0644;


var gTargetFile = null;

const nsICryptoHash = Components.interfaces.nsICryptoHash;
var gHashComp = Components.classes["@mozilla.org/security/hash;1"].
                createInstance(nsICryptoHash);


function initMDHTInfo()
{
  var hashtype_db = document.getElementById("mdht-hashtype-dropbox"); 
  hashtype_db.value = gMDHTUserPreferenceInfo.getPrefValue("default_hash_alg");  

}


function getHash()
{
  var hash_type, checksum;
  var hash_type_db = document.getElementById("mdht-hashtype-dropbox");

  switch(hash_type_db.value)
  {
    case "MD5":  hash_type = nsICryptoHash.MD5;
                 break;

    case "SHA1": hash_type = nsICryptoHash.SHA1;
                 break;

    default:     hash_type = nsICryptoHash.MD5;
                 break;
  }

  var input_type = document.getElementById("inputtype-rgroup").selectedItem.value;

  if (input_type == "file")
   {
    if (gTargetFile)
     {
       var fileStream = Components.classes["@mozilla.org/network/file-input-stream;1"].
                        createInstance(Components.interfaces.nsIFileInputStream);
       fileStream.init(gTargetFile, MODE_RDONLY, PERMS_FILE, 0);

       gHashComp.init(hash_type);

       gHashComp.updateFromStream(fileStream, -1);
       checksum = binaryToHex(gHashComp.finish(false));

       fileStream.close();
     }
   }
  else
   {
     var input_text = document.getElementById("text-input-tbox").value;

     var string_buffer = [];
     for (var i=0; i < input_text.length; i++)       // convert string to array of octets
        string_buffer[i] = input_text.charCodeAt(i); // expected by nsICryptoHash::update()
                                                     
     gHashComp.init(hash_type);

     gHashComp.update(string_buffer, string_buffer.length);
     checksum = binaryToHex(gHashComp.finish(false));
   }

  document.getElementById("mdht-mdigest-desc").value = checksum;
}


function browseMDInputFiles()
{
  var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"]
           .createInstance(nsIFilePicker);

  fp.init(window, "Select a File", nsIFilePicker.modeOpen);
  fp.appendFilters(nsIFilePicker.filterAll);

  var res = fp.show();

  if (res == nsIFilePicker.returnOK)
   {
     gTargetFile = fp.file;
     document.getElementById("file-input-tbox").value = fp.fileURL.spec;
     document.getElementById("mdht-mdigest-desc").value = "";
   }


}



function changeMDInputType(inputTypeStr)
{
  var msg_input_deck = document.getElementById("msg-input-deck");
  msg_input_deck.selectedIndex = ((inputTypeStr == "file") ? 0 : 1);

  document.getElementById("mdht-mdigest-desc").value = "";
}



function copyStringToClipboard(aString)
{
  var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                           .getService(Components.interfaces.nsIClipboardHelper);

  clipboardHelper.copyString(aString);  
}


function compareMessageDigests()
{
  var digestInfo = {};
  var input_type = document.getElementById("inputtype-rgroup").selectedItem.value;

  if (input_type == "file")
   {
     digestInfo.input = document.getElementById("file-input-tbox").value;
     digestInfo.intype = "file"
   }
  else
   {
     digestInfo.input = "\"" + document.getElementById("text-input-tbox").value + "\"";
     digestInfo.intype = "text"
   }

  digestInfo.alg = document.getElementById("mdht-hashtype-dropbox").value;
  digestInfo.md = document.getElementById("mdht-mdigest-desc").value;

  window.openDialog('chrome://mdhashtool/content/dialogCompare.xul',
                    'mdhtCompare','chrome,centerscreen,modal', digestInfo);

}


///////////////////////////////////////////////////////////////////////////////
// Drag and Drop

var gMDHTDNDObserver =
{
  onDragOver: function (aEvent, aFlavour, aDragSession)
  {
    aDragSession.canDrop = true;
  },
  
  onDrop: function(aEvent, aXferData, aDragSession)
  {
    var hash_type, checksum;
    var url = transferUtils.retrieveURLFromData(aXferData.data,
                                                aXferData.flavour.contentType);

    var hash_type_db = document.getElementById("mdht-hashtype-dropbox");

    switch(hash_type_db.value)
    {
      case "MD5":  hash_type = nsICryptoHash.MD5;
                   break;

      case "SHA1": hash_type = nsICryptoHash.SHA1;
                   break;

      default:     hash_type = nsICryptoHash.MD5;
                   break;
    }

    if (aXferData.flavour.contentType == "application/x-moz-file")
     {
       var inputItem = document.getElementById("itype-file-rbtn");
       document.getElementById("inputtype-rgroup").selectedItem = inputItem;
       document.getElementById("file-input-tbox").value = url;
       document.getElementById('msg-input-deck').selectedIndex = 0;

       gTargetFile = aXferData.data;

       var fileStream = Components.classes["@mozilla.org/network/file-input-stream;1"].
                        createInstance(Components.interfaces.nsIFileInputStream);
       fileStream.init(gTargetFile, MODE_RDONLY, PERMS_FILE, 0);

       gHashComp.init(hash_type);

       gHashComp.updateFromStream(fileStream, -1);
       checksum = binaryToHex(gHashComp.finish(false));

       fileStream.close();
     }
    else if (aXferData.flavour.contentType == "text/unicode")
     {
       var inputItem = document.getElementById("itype-text-rbtn");
       document.getElementById("inputtype-rgroup").selectedItem = inputItem;
       document.getElementById("text-input-tbox").value = url;
       document.getElementById('msg-input-deck').selectedIndex = 1;

       var input_text = document.getElementById("text-input-tbox").value;

       var string_buffer = [];
       for (var i=0; i < input_text.length; i++)       // convert string to array of octets
          string_buffer[i] = input_text.charCodeAt(i); // expected by nsICryptoHash::update()
                                                     
       gHashComp.init(hash_type);

       gHashComp.update(string_buffer, string_buffer.length);
       checksum = binaryToHex(gHashComp.finish(false));
     }

    document.getElementById("mdht-mdigest-desc").value = checksum;
  },

  _flavourSet: null,  
  getSupportedFlavours: function ()
  {
    if (!this._flavourSet) {
      this._flavourSet = new FlavourSet();
      this._flavourSet.appendFlavour("application/x-moz-file", "nsIFile");
      this._flavourSet.appendFlavour("text/unicode");
    }
    return this._flavourSet;
  }
}

//
// borrowed from nsUpdateService.js
//
function binaryToHex(input) {
  var result = "";
  for (var i = 0; i < input.length; ++i) {
    var hex = input.charCodeAt(i).toString(16);
    if (hex.length == 1)
      hex = "0" + hex;
    result += hex;
  }
  return result;
}
